// 
// lat - CreateEntryDialog.cs
// Author: Loren Bandiera
// Copyright 2005-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using Novell.Directory.Ldap;
using Gtk;

namespace lat
{
	public class CreateEntryDialog : Gtk.Dialog
	{
		Connection conn;
		ListStore attrListStore;
		List<string> objectClass;
		LdapAttribute objAttr = null;
		Template t;
		bool isTemplate = false;
		bool errorOccured = false;	
	
		protected Gtk.Entry rdnEntry;
		protected Gtk.Button browseButton;
		protected Gtk.TreeView attrTreeView;

		
		public CreateEntryDialog(Connection connection, Template template)
		{
			Stetic.Gui.Build(this, typeof(lat.CreateEntryDialog));
			
			conn = connection;
			t = template;
			isTemplate = true;
			
			foreach (string s in t.Classes) {
				attrListStore.AppendValues ("objectClass", s, "Optional");
				objectClass.Add (s);
			}
			
			ShowAttributes ();
			
			this.Run ();
			while (errorOccured)
				this.Run ();
				
			this.Destroy ();
		}

		public CreateEntryDialog(Connection connection, LdapEntry le)
		{
			Stetic.Gui.Build(this, typeof(lat.CreateEntryDialog));

			conn = connection;

			Init ();

			LdapAttribute la = le.getAttribute ("objectClass");
			foreach (string s in la.StringValueArray) {
				attrListStore.AppendValues ("objectClass", s, "Optional");
				objectClass.Add (s);
			}

			ShowAttributes ();

			this.Run ();
			while (errorOccured)
				this.Run ();

			this.Destroy ();			
		}

		#region internal_methods
		
		void Init ()
		{
			objectClass = new List<string> ();

			attrListStore = new ListStore (typeof (string), typeof (string), typeof (string));
			attrTreeView.Model = attrListStore;
			
			TreeViewColumn col;
			col = attrTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;

			CellRendererText cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnAttributeEdit);

			col = attrTreeView.AppendColumn ("Value", cell, "text", 1);
			col.SortColumnId = 1;

			col = attrTreeView.AppendColumn ("Type", new CellRendererText (), "text", 2);
			col.SortColumnId = 2;

			attrListStore.SetSortColumnId (0, SortType.Ascending);
			
			browseButton.Label = conn.DirectoryRoot;

			this.Resize (320, 200);
		}
		
		void InsertValues (string[] values, string valueType)
		{
			foreach (string s in values) {
				if (s == "objectClass")
					continue;

				if (isTemplate)
					attrListStore.AppendValues (s, t.GetAttributeDefaultValue (s), valueType);
				else
					attrListStore.AppendValues (s, "", valueType);
			}
		}
		
		void ShowAttributes ()
		{
			string[] required, optional;
			conn.Data.GetAllAttributes (objectClass, out required, out optional);

			InsertValues (required, "Required");
			InsertValues (optional, "Optional");
		}
		
		#endregion

		void OnAttributeEdit (object o, EditedArgs args)
		{
			TreeIter iter;
			if (!attrListStore.GetIterFromString (out iter, args.Path))
				return;

			attrListStore.SetValue (iter, 1, args.NewText);		
		}

		protected virtual void OnBrowseClicked(object sender, System.EventArgs e)
		{
			SelectContainerDialog scd = new SelectContainerDialog (conn);

			scd.Message = String.Format (
				Mono.Unix.Catalog.GetString (
				"Where in the directory would\nyou like to save the entry?"));

			scd.Title = Mono.Unix.Catalog.GetString ("Select entry base");
			scd.Run ();

			if (!scd.DN.Equals ("") && !scd.DN.Equals (conn.Settings.Host))
				browseButton.Label = scd.DN;
				
			scd.Destroy ();		
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			string dn = String.Format ("{0},{1}", rdnEntry.Text, browseButton.Label);
			LdapAttributeSet lset = new LdapAttributeSet ();
			
			foreach (object[] row in attrListStore) {
			
				string n = (string) row[0];
				string v = (string) row[1];

				if (n == null || v == null || v == "")
					continue;

				if (n.ToLower() == "objectclass") {
					if (objAttr == null)
						objAttr = new LdapAttribute (n, v);
					else
						objAttr.addValue (v);

				} else {

					LdapAttribute attr = new LdapAttribute (n, v);
					lset.Add (attr);
				}				
			}
			
			lset.Add (objAttr);
			
			LdapEntry entry = new LdapEntry (dn, lset);
			if (!Util.AddEntry (conn, entry))
				errorOccured = true;
			else
				errorOccured = false;		
		}

		protected virtual void OnCancelClicked(object sender, System.EventArgs e)
		{
			errorOccured = false;
		}


	}
}
