// 
// lat - LoginDialog.cs
// Author: Loren Bandiera
// Copyright 2005-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using Novell.Directory.Ldap;

namespace lat
{	
	public class LoginDialog : Gtk.Dialog
	{
		Connection conn;
		bool isRelogin = false;
		string userName;
		string userPass;
	
		protected Gtk.Label msgLabel;
		protected Gtk.Entry userEntry;
		protected Gtk.Entry passEntry;
		protected Gtk.CheckButton useSSLCheckButton;

		
		public LoginDialog(string userName)
		{
			Stetic.Gui.Build(this, typeof(lat.LoginDialog));
			
			msgLabel.Text = Mono.Unix.Catalog.GetString ("Enter your password");
			userEntry.Text = userName;
			useSSLCheckButton.HideAll ();
			
		}

		public LoginDialog(string msg, string userName)
		{
			Stetic.Gui.Build(this, typeof(lat.LoginDialog));
			
			msgLabel.Text = msg;
			userEntry.Text = userName;
			isRelogin = true;
		}

		public LoginDialog (Connection connection, string msg)
		{
			Stetic.Gui.Build(this, typeof(lat.LoginDialog));

			conn = connection;
			msgLabel.Text = msg;
			isRelogin = true;
		}

		public new bool Run ()
		{
			ResponseType res = (ResponseType) base.Run ();
			this.Destroy ();
			
			if (res == ResponseType.Ok)
				return true;
			
			return false;
		}

		public string UserName
		{
			get { return userName; }
		}

		public string UserPass
		{
			get { return userPass; }
		}

		#region internal_methods
		
		void Relogin ()
		{
			try {
			
				if (useSSLCheckButton.Active)
					conn.Settings.Encryption = EncryptionType.SSL;

				if (conn.Settings.Encryption == EncryptionType.TLS)
					conn.StartTLS ();

				conn.Bind (userEntry.Text, passEntry.Text);

			} catch (Exception e) {

				string errorMsg =
					Mono.Unix.Catalog.GetString ("Unable to re-login");

				errorMsg += "\nError: " + e.Message;

				HIGMessageDialog dialog = new HIGMessageDialog (
					this,
					0,
					Gtk.MessageType.Error,
					Gtk.ButtonsType.Ok,
					"Login error",
					errorMsg);

				dialog.Run ();
				dialog.Destroy ();
			}
		}
		
		#endregion

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			if (isRelogin) {
				Relogin ();
			} else {
				userName = userEntry.Text;
				userPass = passEntry.Text;
			}
		}
	}
}
