// 
// lat - MassEditDialog.cs
// Author: Loren Bandiera
// Copyright 2005-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using Novell.Directory.Ldap;

namespace lat
{	
	public class MassEditDialog : Gtk.Dialog
	{
		ListStore modListStore;
		List<LdapModification> modList;

		ComboBox connComboBox;
		ComboBox actionComboBox;
		
		protected Gtk.HBox serverHBox;
		protected Gtk.HBox actionHBox;
		protected Gtk.TreeView modListView;
		protected Gtk.Entry valueEntry;
		protected Gtk.Entry nameEntry;
		protected Gtk.Entry searchEntry;

		
		public MassEditDialog()
		{
			Stetic.Gui.Build(this, typeof(lat.MassEditDialog));
			
			modList = new List<LdapModification> ();
			
			CreateCombos ();
			
			modListStore = new ListStore (typeof (string), typeof (string), typeof (string));
			modListView.Model = modListStore;
			
			TreeViewColumn col;
			col = modListView.AppendColumn ("Action", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;

			col = modListView.AppendColumn ("Name", new CellRendererText (), "text", 1);
			col.SortColumnId = 1;

			col = modListView.AppendColumn ("Value", new CellRendererText (), "text", 2);
			col.SortColumnId = 2;

			modListStore.SetSortColumnId (0, SortType.Ascending);

			this.Resize (300, 450);
		}

		#region internal_methods
		
		void CreateCombos ()
		{
			// class
			actionComboBox = ComboBox.NewText ();
			actionComboBox.AppendText ("Add");
			actionComboBox.AppendText ("Delete");
			actionComboBox.AppendText ("Replace");

			actionComboBox.Active = 0;
			actionComboBox.Show ();

			actionHBox.PackStart (actionComboBox, true, true, 0);
			
			connComboBox = Util.CreateServerCombo ();
			serverHBox.PackStart (connComboBox, true, true, 5);
		}
		
		Connection GetCurrentConnection ()
		{
			TreeIter iter;
			if (!connComboBox.GetActiveIter (out iter))
				return null;

			string name = (string) connComboBox.Model.GetValue (iter, 0);
			
			Connection conn = Global.Connections [name];
			if (!conn.IsConnected)
				conn.Connect ();			
			
			return conn;
		}
		
		#endregion

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			Connection conn = GetCurrentConnection ();
		
			LdapEntry[] sr = conn.Data.Search (conn.DirectoryRoot, searchEntry.Text);
			
			foreach (object[] row in modListStore) {

				string _action = (string) row[0];
				string _name = (string) row[1];
				string _value = (string) row[2];
				
				LdapAttribute a = new LdapAttribute (_name, _value);
				LdapModification m = null;

				switch (_action.ToLower()) {

				case "add":
					m = new LdapModification (LdapModification.ADD, a);
					break;

				case "delete":
					m = new LdapModification (LdapModification.DELETE, a);
					break;

				case "replace":
					m = new LdapModification (LdapModification.REPLACE, a);
					break;

				default:
					break;
				}

				if (m != null)
					modList.Add (m);		
			}
			
			foreach (LdapEntry le in sr)
				Util.ModifyEntry (conn, le.DN, modList.ToArray());
		}

		protected virtual void OnAddClicked(object sender, System.EventArgs e)
		{
			TreeIter iter;
			if (!actionComboBox.GetActiveIter (out iter))
				return;

			string action = (string) actionComboBox.Model.GetValue (iter, 0);
			
			if (nameEntry.Text != "" && valueEntry.Text != "")
				modListStore.AppendValues (action, nameEntry.Text, valueEntry.Text);		
		}

		protected virtual void OnRemoveClicked(object sender, System.EventArgs e)
		{
			Gtk.TreeIter iter;
			Gtk.TreeModel model;
			
			if (modListView.Selection.GetSelected (out model, out iter)) 
				modListStore.Remove (ref iter);
		}

		protected virtual void OnClearClicked(object sender, System.EventArgs e)
		{
			modListStore.Clear ();
			modList.Clear ();		
		}

		protected virtual void OnSearchBuilderClicked(object sender, System.EventArgs e)
		{
			SearchBuilderDialog sbd = new SearchBuilderDialog ();
			sbd.Run ();
			
			searchEntry.Text = sbd.UserFilter;
			
			sbd.Destroy ();
		}
	}
}
