// 
// lat - PasswordDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Security.Cryptography;
using System.Text;

using Gtk;
using Mono.Unix;

namespace lat
{	
	public class PasswordDialog : Gtk.Dialog
	{
		string unixPassword;
		string lmPassword;
		string ntPassword;
		bool passwordsMatch;
	
		protected Gtk.Entry passwordEntry;
		protected Gtk.Entry reenterEntry;
		protected Gtk.RadioButton md5RadioButton;
		protected Gtk.RadioButton shaRadioButton;
		protected Gtk.CheckButton useSaltCheckButton;

		
		public PasswordDialog()
		{
			Stetic.Gui.Build(this, typeof(lat.PasswordDialog));
			
			shaRadioButton.Active = true;
			useSaltCheckButton.Active = true;
			passwordsMatch = true;
		}

		public bool PasswordsMatch
		{
			get { return passwordsMatch; }
			set { passwordsMatch = value; }
		}

		public string UnixPassword
		{
			get { return unixPassword; }
		}

		public string LMPassword
		{
			get { return lmPassword; }
		}

		public string NTPassword
		{
			get { return ntPassword; }
		}

		#region internal_methods

		byte[] AddSalt (HashAlgorithm hashAlgorithm, byte[] buffer)
		{
			// get salt
			byte[] saltBytes = GetSalt ();

			// Create buffer for password + salt
		        byte[] bufferWithSaltBytes = 
       	   		     new byte[buffer.Length + saltBytes.Length];

			// Insert password
			for (int i = 0; i < buffer.Length; i++)
				bufferWithSaltBytes[i] = buffer[i];

			// Insert salt
			for (int i = 0; i < saltBytes.Length; i++)
				bufferWithSaltBytes[buffer.Length + i] = saltBytes[i];

			// Encrypt
			byte[] hashBytes = hashAlgorithm.ComputeHash (bufferWithSaltBytes);

			// Create byte array for encrypted hash + salt
			byte[] hashWithSaltBytes = new byte[hashBytes.Length + 
				saltBytes.Length];

			// Insert hashed password
			for (int i = 0; i < hashBytes.Length; i++)
				hashWithSaltBytes[i] = hashBytes[i];

			// Insert salt
			for (int i = 0; i < saltBytes.Length; i++)
				hashWithSaltBytes[hashBytes.Length + i] = saltBytes[i];

			return hashWithSaltBytes;
		}
				
		string DoEncryption (string input, string algorithm, bool salted)
		{
			string retVal = "";
			if (input == string.Empty)
				return retVal;

			HashAlgorithm hashAlgorithm = null;
			byte[] hash = null;
			string encText = null;

			switch (algorithm) {

			case "MD5":
				hashAlgorithm = new MD5CryptoServiceProvider();
				encText = "{MD5}";
				break;

			case "SHA":
				hashAlgorithm = new SHA1Managed();
				encText = "{SHA}";
				break;
			}

			ASCIIEncoding enc = new ASCIIEncoding();

			byte[] buffer = enc.GetBytes (input);

			if (salted) {

				byte[] saltedBuffer = AddSalt (hashAlgorithm, buffer);
				encText = "{S" + algorithm + "}";         
				retVal = encText + Convert.ToBase64String (saltedBuffer);

			} else {

				hash = hashAlgorithm.ComputeHash(buffer);
				retVal = encText + Convert.ToBase64String (hash);
			}

			return retVal;
		}

		void GeneratePassword ()
		{	 
			if (md5RadioButton.Active)
				unixPassword = DoEncryption (passwordEntry.Text, "MD5", useSaltCheckButton.Active);
			else
				unixPassword = DoEncryption (passwordEntry.Text, "SHA", useSaltCheckButton.Active);
			
			SMBPassword smbpass = new SMBPassword (passwordEntry.Text);
			lmPassword = smbpass.LM;
			ntPassword = smbpass.NT;
		}

		byte[] GetSalt ()
		{
			byte[] retVal = new byte [8];

			RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
			rng.GetNonZeroBytes (retVal); 

			return retVal;
		}
		
		#endregion
		
		protected virtual void OnCancelClicked(object sender, System.EventArgs e)
		{
			unixPassword = null;
		}

		protected virtual void OnEncryptionToggled(object sender, System.EventArgs e)
		{
			GeneratePassword ();
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			if (passwordEntry.Text != reenterEntry.Text) {
				passwordsMatch = false;
				return;
			}
			
			GeneratePassword ();
		}
	}
}
