// 
// lat - PluginPreferencesDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using Gtk;

namespace lat
{
	public class PluginPreferencesDialog : Gtk.Dialog
	{
		ListStore columnStore;
		Connection conn;
		ViewPlugin vp;
		List<string> colNames;
		List<string> colAttrs;	
	
		protected Gtk.TreeView columnsTreeView;
		protected Gtk.Entry filterEntry;
		protected Gtk.Button searchBaseButton;
		protected Gtk.Button newContainerButton;

		
		public PluginPreferencesDialog(Connection connection, string pluginName)
		{
			Stetic.Gui.Build(this, typeof(lat.PluginPreferencesDialog));
			
			conn = connection;
			colNames = new List<string> ();
			colAttrs = new List<string> ();
			
			columnStore = new ListStore (typeof (string), typeof (string));
			
			CellRendererText cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnNameEdit);			
			columnsTreeView.AppendColumn ("Name", cell, "text", 0);
			
			cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnAttributeEdit);			
			columnsTreeView.AppendColumn ("Attribute", cell, "text", 1);
			
			columnsTreeView.Model = columnStore;
			
			vp = Global.Plugins.GetViewPlugin (pluginName, connection.Settings.Name); 
			if (vp != null) {			
				for (int i = 0; i < vp.ColumnNames.Length; i++) {  
					columnStore.AppendValues (vp.ColumnNames[i], vp.ColumnAttributes[i]);
					colNames.Add (vp.ColumnNames[i]);
					colAttrs.Add (vp.ColumnAttributes[i]);
				}
					
				filterEntry.Text = vp.Filter;
					
				if (vp.DefaultNewContainer != "")
					newContainerButton.Label = vp.DefaultNewContainer;

				if (vp.SearchBase != "")
					searchBaseButton.Label = vp.SearchBase;					
			} else {
				Log.Error ("Unable to find view plugin {0}", pluginName); 
			}
			
			this.Resize (300, 400);
		}

		void OnAttributeEdit (object o, EditedArgs args)
		{
			TreeIter iter;
			if (!columnStore.GetIterFromString (out iter, args.Path))
				return;
				
			string oldAttr = (string) columnStore.GetValue (iter, 1);		
			colAttrs.Remove (oldAttr);
			colAttrs.Add (args.NewText);
			
			columnStore.SetValue (iter, 1, args.NewText);
		}
		
		void OnNameEdit (object o, EditedArgs args)
		{
			TreeIter iter;
			if (!columnStore.GetIterFromString (out iter, args.Path))
				return;
				
			string oldName = (string) columnStore.GetValue (iter, 0);		
			colNames.Remove (oldName);	
			colNames.Add (args.NewText);
			
			columnStore.SetValue (iter, 0, args.NewText);
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			ViewPluginConfig vpc = new ViewPluginConfig ();
			vpc.ColumnAttributes = colAttrs.ToArray ();
			vpc.ColumnNames = colNames.ToArray ();
			vpc.PluginName = vp.Name;
			vpc.Filter = filterEntry.Text;

			if (newContainerButton.Label != "")
				vpc.DefaultNewContainer = newContainerButton.Label;

			if (searchBaseButton.Label != "")
				vpc.SearchBase = searchBaseButton.Label;			

			if (vp.PluginConfiguration.Defaults == null)		
				vpc.Defaults = new Dictionary<string,string> ();
			else
				vpc.Defaults = vp.PluginConfiguration.Defaults;
						
			vp.PluginConfiguration = vpc;
			Global.Plugins.SetPluginConfiguration (conn.Settings.Name, vpc);		
		}

		protected virtual void OnFilterBuildClicked(object sender, System.EventArgs e)
		{
			SearchBuilderDialog sbd = new SearchBuilderDialog ();
			sbd.Run ();
			filterEntry.Text = sbd.UserFilter;
			sbd.Destroy ();
		}

		protected virtual void OnAddClicked(object sender, System.EventArgs e)
		{
			columnStore.AppendValues ("Untitiled", "Unknown");
		}

		protected virtual void OnRemoveClicked(object sender, System.EventArgs e)
		{
			TreeModel model;
			TreeIter iter;

			if (columnsTreeView.Selection.GetSelected (out model, out iter)) {
				string name = (string) columnStore.GetValue (iter, 0);
				string attr = (string) columnStore.GetValue (iter, 1);
				colNames.Remove (name);
				colAttrs.Remove (attr);
				columnStore.Remove (ref iter);
			}		
		}

		protected virtual void OnSearchBaseClicked(object sender, System.EventArgs e)
		{
			SelectContainerDialog scd = new SelectContainerDialog (conn);
			scd.Message = String.Format (Mono.Unix.Catalog.GetString ("Select a search base"));
			scd.Title = Mono.Unix.Catalog.GetString ("Select container");
			scd.Run ();

			if (!scd.DN.Equals ("") && !scd.DN.Equals (conn.Settings.Host))
				searchBaseButton.Label = scd.DN;
				
			scd.Destroy ();
		}

		protected virtual void OnNewContainerClicked(object sender, System.EventArgs e)
		{
			SelectContainerDialog scd = new SelectContainerDialog (conn);
			scd.Message = String.Format (Mono.Unix.Catalog.GetString ("Select a container for new objects"));
			scd.Title = Mono.Unix.Catalog.GetString ("Select container");
			scd.Run ();

			if (!scd.DN.Equals ("") && !scd.DN.Equals (conn.Settings.Host))
				newContainerButton.Label = scd.DN;
				
			scd.Destroy ();		
		}

		protected virtual void OnSetDefaultValuesClicked(object sender, System.EventArgs e)
		{
			vp.OnSetDefaultValues (conn);
			Log.Debug ("OnSetDefaultValuesClicked vp.PluginConfiguration.Defaults.Count: {0}", vp.PluginConfiguration.Defaults.Count);		
		}
	}
}
