// 
// lat - PreferencesDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using Gtk;

namespace lat
{
	public class PreferencesDialog : Gtk.Dialog
	{
		ListStore profileStore;
		Gnome.Program program;
		bool gettingHelp = false;	
	
		protected Gtk.RadioButton browserDoubleClickButton;
		protected Gtk.RadioButton browserSingleClickButton;
		protected Gtk.TreeView profilesTreeView;

		
		public PreferencesDialog(Gnome.Program program)
		{
			Stetic.Gui.Build(this, typeof(lat.PreferencesDialog));
			
			profileStore = new ListStore (typeof (string));
			profilesTreeView.Model = profileStore;
			profileStore.SetSortColumnId (0, SortType.Ascending);
			
			TreeViewColumn col;
			col = profilesTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;

			UpdateProfileList ();
					
			LoadPreference (Preferences.BROWSER_SELECTION);
			
			this.Resize (300, 400);			
			this.Run ();
			
			if (gettingHelp) {
				this.Run ();
				gettingHelp = false;
			}
			
			this.Destroy ();			
		}

		#region internal_methods
		
		string GetSelectedProfileName ()
		{
			TreeIter iter;
			TreeModel model;

			if (profilesTreeView.Selection.GetSelected (out model, out iter))  {
				string name = (string) model.GetValue (iter, 0);
				return name;
			}

			return null;
		}
		
		void LoadPreference (String key)
		{
			object val = Preferences.Get (key);

			if (val == null)
				return;
			
			switch (key) {
				
			case Preferences.BROWSER_SELECTION:
				int b = (int) val;
				if (b == 1)
					browserSingleClickButton.Active = true;
				else if (b == 2)
					browserDoubleClickButton.Active = true;
					
				break;
			}
		}
		
		void UpdateProfileList ()
		{
			profileStore.Clear ();
			
			foreach (string s in Global.Connections.ConnectionNames)
				profileStore.AppendValues (s);
		}
		
		#endregion

		protected virtual void OnDoubleClickToggled(object sender, System.EventArgs e)
		{
			if (browserSingleClickButton.Active)
				Preferences.Set (Preferences.BROWSER_SELECTION, 1);
			else
				Preferences.Set (Preferences.BROWSER_SELECTION, 2);		
		}

		protected virtual void OnHelpClicked(object sender, System.EventArgs e)
		{
			try {

				gettingHelp = true;

				Gnome.Help.DisplayDesktopOnScreen (program, 
					Defines.PACKAGE, 
					"lat.xml", 
					"lat-preferences", 
					Gdk.Screen.Default);

			} catch (Exception ex) {

				Log.Debug (ex);
				Util.ShowErrorDialog ("Help error", ex.Message);
			}		
		}

		protected virtual void OnAddProfileClicked(object sender, System.EventArgs e)
		{
			ProfilePreferencesDialog dlg = new ProfilePreferencesDialog ();
			dlg.Run ();
			dlg.Destroy ();
			
			UpdateProfileList ();
		}

		protected virtual void OnEditProfileClicked(object sender, System.EventArgs e)
		{
			string profileName = GetSelectedProfileName ();
			if (profileName == null)
				return;
			
			Connection conn = Global.Connections [profileName];
			
			ProfilePreferencesDialog dlg = new ProfilePreferencesDialog (conn);
			dlg.Run ();
			dlg.Destroy ();
			
			UpdateProfileList ();		
		}

		protected virtual void OnDeleteProfileClicked(object sender, System.EventArgs e)
		{
			string profileName = GetSelectedProfileName ();
			string msg = null;
			
			if (profileName == null) {
				// show dialog; must select entry to edit
				return;
			}

			msg = String.Format ("{0} {1}",
				Mono.Unix.Catalog.GetString (
				"Are you sure you want to delete the profile:"),
				profileName);
				
			if (Util.AskYesNo (this, msg)) {
				Global.Connections.Delete (profileName);
				Global.Connections.Save ();
				UpdateProfileList ();				
			}		
		}
	}
}
