// 
// lat - EditAdComputerViewDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using Novell.Directory.Ldap;
using lat;

namespace AD.Gui
{	
	public class EditAdComputerViewDialog : Gtk.Dialog
	{
		protected Connection conn;		
		protected bool missingValues;
		protected bool errorOccured;
		protected string defaultNewContainer;		
		protected LdapEntry currentEntry;
	
		protected Gtk.Entry computerNameEntry;
		protected Gtk.Entry dnsNameEntry;
		protected Gtk.Entry descriptionEntry;
		protected Gtk.Entry osNameEntry;
		protected Gtk.Entry osVersionEntry;
		protected Gtk.Entry osServicePackEntry;
		protected Gtk.Entry locationEntry;
		protected Gtk.Label manCityLabel;
		protected Gtk.Label manStateLabel;
		protected Gtk.Label manCountryLabel;
		protected Gtk.Label manTelephoneNumberLabel;
		protected Gtk.Label manFaxNumberLabel;
		protected Gtk.Entry manNameEntry;
		protected Gtk.TextView manStreetTextView;
		protected Gtk.Label manOfficeLabel;

		
		public EditAdComputerViewDialog(Connection connection, LdapEntry le)
		{
			Stetic.Gui.Build(this, typeof(AD.Gui.EditAdComputerViewDialog));
			
			this.conn = connection;
			this.currentEntry = le;
			
			string cpName = (string) conn.Data.GetAttributeValueFromEntry (currentEntry, "cn");
			computerNameEntry.Text = cpName.ToUpper();

			this.Title = cpName + " Properties";

			dnsNameEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "dNSHostName");
			descriptionEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "description");
			
			osNameEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "operatingSystem");
			osVersionEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "operatingSystemVersion");
			osServicePackEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "operatingSystemServicePack");

			locationEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "location");

			string manName = conn.Data.GetAttributeValueFromEntry (currentEntry, "managedBy");
			manNameEntry.Text = manName;

			if (manName != "" || manName != null)
				UpdateManagedBy (manName);

			this.Run ();

			while (missingValues || errorOccured) {
				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();				
			}

			this.Destroy ();
		}

		#region internal_methods
		
		LdapEntry CreateEntry (string dn)
		{
			LdapAttributeSet aset = new LdapAttributeSet();
			aset.Add (new LdapAttribute ("objectClass", new string[] {"computer"}));
			aset.Add (new LdapAttribute ("cn", computerNameEntry.Text.ToLower()));
			aset.Add (new LdapAttribute ("description", descriptionEntry.Text));
			aset.Add (new LdapAttribute ("dNSHostName", dnsNameEntry.Text));
			aset.Add (new LdapAttribute ("operatingSystem", osNameEntry.Text));
			aset.Add (new LdapAttribute ("operatingSystemVersion", osVersionEntry.Text));
			aset.Add (new LdapAttribute ("operatingSystemServicePack", osServicePackEntry.Text));
			aset.Add (new LdapAttribute ("location", locationEntry.Text));
			aset.Add (new LdapAttribute ("managedBy", manNameEntry.Text));
								
			LdapEntry newEntry = new LdapEntry (dn, aset);
			return newEntry;
		}
		
		void UpdateManagedBy (string dn)
		{
			try {

				LdapEntry leMan = conn.Data.GetEntry (dn);

				manOfficeLabel.Text = conn.Data.GetAttributeValueFromEntry (
					leMan, "physicalDeliveryOfficeName");

				manStreetTextView.Buffer.Text = conn.Data.GetAttributeValueFromEntry 
					(leMan, "streetAddress");

				manCityLabel.Text = conn.Data.GetAttributeValueFromEntry (
					leMan, "l");

				manStateLabel.Text = conn.Data.GetAttributeValueFromEntry (
					leMan, "st");

				manCountryLabel.Text = conn.Data.GetAttributeValueFromEntry (
					leMan, "c");

				manTelephoneNumberLabel.Text = conn.Data.GetAttributeValueFromEntry 
					(leMan, "telephoneNumber");

				manFaxNumberLabel.Text = conn.Data.GetAttributeValueFromEntry (
					leMan, "facsimileTelephoneNumber");

			} catch {

				manOfficeLabel.Text = "";
				manStreetTextView.Buffer.Text = "";
				manCityLabel.Text = "";
				manStateLabel.Text = "";
				manCountryLabel.Text = "";
				manTelephoneNumberLabel.Text = "";
				manFaxNumberLabel.Text = "";
			}
		}
		
		#endregion

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			LdapEntry entry = null;			
			entry = CreateEntry (currentEntry.DN);				 
				 
			LdapEntryAnalyzer lea = new LdapEntryAnalyzer ();
			lea.Run (currentEntry, entry);
				 
			if (lea.Differences.Length == 0)
				return;
				 	
			if (!Util.ModifyEntry (conn, entry.DN, lea.Differences))
				errorOccured = true;
		}

		protected virtual void OnManClearClicked(object sender, System.EventArgs e)
		{
			manNameEntry.Text = "";
			UpdateManagedBy ("none");
		}

		protected virtual void OnManChangedClicked(object sender, System.EventArgs e)
		{
			SelectContainerDialog scd = new SelectContainerDialog (conn);
			scd.Title = "Save Computer";
			scd.Message = Mono.Unix.Catalog.GetString (
					"Select a user who will manage ") + 
					computerNameEntry.Text.ToLower();

			scd.Run ();

			if (scd.DN == "") {
				return;
			} else {
				manNameEntry.Text = scd.DN;
				UpdateManagedBy (scd.DN);
			}
			
			scd.Destroy ();
		}
	}
}
