// 
// lat - EditContactsViewDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using Novell.Directory.Ldap;
using lat;

namespace AD.Gui
{
	public class EditContactsViewDialog : Gtk.Dialog
	{
		protected Connection conn;		
		protected bool missingValues;
		protected bool errorOccured;
		protected string defaultNewContainer;
		protected LdapEntry currentEntry;
		
		protected Gtk.Label gnNameLabel;
		protected Gtk.Entry gnFirstNameEntry;
		protected Gtk.Entry gnInitialsEntry;
		protected Gtk.Entry gnLastNameEntry;
		protected Gtk.Entry gnDisplayNameEntry;
		protected Gtk.Entry gnDescriptionEntry;
		protected Gtk.Entry gnOfficeEntry;
		protected Gtk.Entry gnTelephoneNumberEntry;
		protected Gtk.Entry gnEmailEntry;
		protected Gtk.Entry gnWebPageEntry;
		protected Gtk.TextView adStreetTextView;
		protected Gtk.Entry adPOBoxEntry;
		protected Gtk.Entry adCityEntry;
		protected Gtk.Entry adStateEntry;
		protected Gtk.Entry adZipEntry;
		protected Gtk.Entry adCountryEntry;
		protected Gtk.Entry tnHomeEntry;
		protected Gtk.Entry tnPagerEntry;
		protected Gtk.Entry tnMobileEntry;
		protected Gtk.Entry tnFaxEntry;
		protected Gtk.Entry tnIPPhoneEntry;
		protected Gtk.TextView tnNotesTextView;
		protected Gtk.Entry ozTitleEntry;
		protected Gtk.Entry ozDeptEntry;
		protected Gtk.Entry ozCompanyEntry;

		
		public EditContactsViewDialog(Connection connection, LdapEntry le)
		{
			Stetic.Gui.Build(this, typeof(AD.Gui.EditContactsViewDialog));
			
			this.conn = connection;
			this.currentEntry = le;
			
			string displayName = conn.Data.GetAttributeValueFromEntry (currentEntry, "displayName");

			gnNameLabel.Text = displayName;
			gnFirstNameEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "givenName");
			gnInitialsEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "initials");
			gnLastNameEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "sn");
			gnDisplayNameEntry.Text = displayName;
			gnDescriptionEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "description");
			gnOfficeEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "physicalDeliveryOfficeName");
			gnTelephoneNumberEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "telephoneNumber");
			gnEmailEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "mail");
			
			adPOBoxEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "postOfficeBox");
			adCityEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "l");
			adStateEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "st");
			adZipEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "postalCode");
			

			tnHomeEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "homePhone");
			tnPagerEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "pager");
			tnMobileEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "mobile");
			tnFaxEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "facsimileTelephoneNumber");
			
			ozTitleEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "title");

			string contactName = conn.Data.GetAttributeValueFromEntry (currentEntry, "cn");
			this.Title = contactName + " Properties";

			gnWebPageEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "wWWHomePage");

			adStreetTextView.Buffer.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "streetAddress");
			adCountryEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "co");

			tnIPPhoneEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "ipPhone");
			tnNotesTextView.Buffer.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "info");

			ozDeptEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "department");
			ozCompanyEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "company");

			this.Run ();

			while (missingValues || errorOccured) {

				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();
			}

			this.Destroy ();
		}

		LdapEntry CreateEntry (string dn)
		{
			LdapAttributeSet aset = new LdapAttributeSet();
			aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "person", "organizationalPerson", "contact" }));
			aset.Add (new LdapAttribute ("givenName", gnFirstNameEntry.Text));
			aset.Add (new LdapAttribute ("initials", gnInitialsEntry.Text));
			aset.Add (new LdapAttribute ("sn", gnLastNameEntry.Text));
			aset.Add (new LdapAttribute ("displayName", gnDisplayNameEntry.Text));
			aset.Add (new LdapAttribute ("cn", gnDisplayNameEntry.Text));
			aset.Add (new LdapAttribute ("wWWHomePage", gnWebPageEntry.Text));
			aset.Add (new LdapAttribute ("physicalDeliveryOfficeName", gnOfficeEntry.Text));
			aset.Add (new LdapAttribute ("mail", gnEmailEntry.Text));
			aset.Add (new LdapAttribute ("description", gnDescriptionEntry.Text));
			aset.Add (new LdapAttribute ("street", adStreetTextView.Buffer.Text));
			aset.Add (new LdapAttribute ("l", adCityEntry.Text));
			aset.Add (new LdapAttribute ("st", adStateEntry.Text));
			aset.Add (new LdapAttribute ("postalCode", adZipEntry.Text));
			aset.Add (new LdapAttribute ("postOfficeBox", adPOBoxEntry.Text));
			aset.Add (new LdapAttribute ("co", adCountryEntry.Text));
			aset.Add (new LdapAttribute ("telephoneNumber", gnTelephoneNumberEntry.Text));
			aset.Add (new LdapAttribute ("facsimileTelephoneNumber", tnFaxEntry.Text));
			aset.Add (new LdapAttribute ("pager", tnPagerEntry.Text));
			aset.Add (new LdapAttribute ("mobile", tnMobileEntry.Text));
			aset.Add (new LdapAttribute ("homePhone", tnHomeEntry.Text));
			aset.Add (new LdapAttribute ("ipPhone", tnIPPhoneEntry.Text));
			aset.Add (new LdapAttribute ("title", ozTitleEntry.Text));
			aset.Add (new LdapAttribute ("department", ozDeptEntry.Text));
			aset.Add (new LdapAttribute ("company", ozCompanyEntry.Text));
			aset.Add (new LdapAttribute ("streetAddress", adStreetTextView.Buffer.Text));
			aset.Add (new LdapAttribute ("info", tnNotesTextView.Buffer.Text));
			
			LdapEntry newEntry = new LdapEntry (dn, aset);
			return newEntry;
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			LdapEntry entry = null;
			
			 entry = CreateEntry (currentEntry.DN);				 
				 
			 LdapEntryAnalyzer lea = new LdapEntryAnalyzer ();
			 lea.Run (currentEntry, entry);
				 
			 if (lea.Differences.Length == 0)
			 	return;
				 	
			 if (!Util.ModifyEntry (conn, entry.DN, lea.Differences))
			 	errorOccured = true;
		}

		protected virtual void OnNameChanged(object sender, System.EventArgs e)
		{
			if (gnInitialsEntry.Text != "")
				gnNameLabel.Text = String.Format ("{0} {1} {2}", gnFirstNameEntry.Text, gnInitialsEntry.Text, gnLastNameEntry.Text);
			else
				gnNameLabel.Text = String.Format ("{0} {1}", gnFirstNameEntry.Text, gnLastNameEntry.Text);
				
			gnDisplayNameEntry.Text = gnNameLabel.Text; 		
		}
	}
}
