// 
// lat - NewContactsViewDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using lat;
using Novell.Directory.Ldap;

namespace AD.Gui
{	
	public class NewContactsViewDialog : Gtk.Dialog
	{
		protected Connection conn;		
		protected bool missingValues;
		protected bool errorOccured;
		protected string defaultNewContainer;
		
		protected Gtk.Label gnNameLabel;
		protected Gtk.Entry gnFirstNameEntry;
		protected Gtk.Entry gnInitialsEntry;
		protected Gtk.Entry gnLastNameEntry;
		protected Gtk.Entry gnDisplayNameEntry;

		
		public NewContactsViewDialog(Connection connection, string newContainer)
		{
			Stetic.Gui.Build(this, typeof(AD.Gui.NewContactsViewDialog));
			
			this.Title = "New contact";
			this.Run ();

			while (missingValues || errorOccured) {

				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();				
			}

			this.Destroy ();			
		}

		#region internal_methods
		
		LdapEntry CreateEntry (string dn)
		{
			LdapAttributeSet aset = new LdapAttributeSet();
			aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "person", "inetOrgPerson" }));
			aset.Add (new LdapAttribute ("givenName", gnFirstNameEntry.Text));
			aset.Add (new LdapAttribute ("initials", gnInitialsEntry.Text));
			aset.Add (new LdapAttribute ("sn", gnLastNameEntry.Text));
			aset.Add (new LdapAttribute ("displayName", gnDisplayNameEntry.Text));
			aset.Add (new LdapAttribute ("cn", gnDisplayNameEntry.Text));
					
			LdapEntry newEntry = new LdapEntry (dn, aset);
			return newEntry;
		}
		
		#endregion

		protected virtual void OnNameChanged(object sender, System.EventArgs e)
		{
			if (gnInitialsEntry.Text != "")
				gnNameLabel.Text = String.Format ("{0} {1} {2}", gnFirstNameEntry.Text, gnInitialsEntry.Text, gnLastNameEntry.Text);
			else
				gnNameLabel.Text = String.Format ("{0} {1}", gnFirstNameEntry.Text, gnLastNameEntry.Text);
				
			gnDisplayNameEntry.Text = gnNameLabel.Text; 
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			LdapEntry entry = null;			
			string userDN = null;
			
			if (this.defaultNewContainer == null) {
			
				SelectContainerDialog scd =	new SelectContainerDialog (conn);
				scd.Title = "Save Group";
				scd.Message = String.Format ("Where in the directory would you like save the contact {0}?", gnDisplayNameEntry.Text);
				scd.Run ();

				if (scd.DN == "")
					return;

				userDN = String.Format ("cn={0},{1}", gnDisplayNameEntry.Text, scd.DN);
				scd.Destroy ();
			
			} else {
			
				userDN = String.Format ("cn={0},{1}", gnDisplayNameEntry.Text, this.defaultNewContainer);
			}
			
			entry = CreateEntry (userDN);

			string[] missing = LdapEntryAnalyzer.CheckRequiredAttributes (conn, entry);
			if (missing.Length != 0) {
				Util.ShowMissingAttrbutesAlert (missing);
				missingValues = true;
				return;
			}

			if (!Util.AddEntry (conn, entry))
				errorOccured = true;
		}
	}
}
