// 
// lat - EditUserViewDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Text;
using Novell.Directory.Ldap;

namespace lat
{	
	public class EditUserViewDialog : Gtk.Dialog
	{
		protected Connection conn;
		protected bool missingValues;
		protected bool errorOccured;
		protected string defaultNewContainer;

		bool isSamba = false;
		bool firstTimeSamba = false;
		string pass = "";
		string smbLM = "";
		string smbNT = "";
		string smbSID = "";
		bool passChanged = false;
		
		LdapEntry currentEntry;

		Dictionary<string,LdapEntry> _allGroups;
		Dictionary<string,string> _allGroupGids;
		Dictionary<string,LdapModification> _modsGroup;
		Dictionary<string,string> _memberOfGroups;

		ListStore _memberOfStore;
	
		protected Gtk.Label usernameLabel;
		protected Gtk.Label fullnameLabel;
		protected Gtk.Entry firstNameEntry;
		protected Gtk.Entry initialsEntry;
		protected Gtk.Entry lastNameEntry;
		protected Gtk.Entry descriptionEntry;
		protected Gtk.Entry officeEntry;
		protected Gtk.Entry phoneEntry;
		protected Gtk.Entry mailEntry;
		protected Gtk.Entry usernameEntry;
		protected Gtk.SpinButton uidSpinButton;
		protected Gtk.Entry homeDirEntry;
		protected Gtk.Entry shellEntry;
		protected Gtk.CheckButton smbEnableSambaButton;
		protected Gtk.Entry smbHomeDriveEntry;
		protected Gtk.Entry smbHomePathEntry;
		protected Gtk.Entry smbLoginScriptEntry;
		protected Gtk.Entry smbProfilePathEntry;
		protected Gtk.Entry smbExpireEntry;
		protected Gtk.Entry smbCanChangePwdEntry;
		protected Gtk.Entry smbMustChangePwdEntry;
		protected Gtk.Label primaryGroupLabel;
		protected Gtk.TreeView memberOfTreeview;
		protected Gtk.TextView adStreetTextView;
		protected Gtk.Entry adPOBoxEntry;
		protected Gtk.Entry adCityEntry;
		protected Gtk.Entry adStateEntry;
		protected Gtk.Entry adZipEntry;
		protected Gtk.Entry tnHomeEntry;
		protected Gtk.Entry tnPagerEntry;
		protected Gtk.Entry tnMobileEntry;
		protected Gtk.Entry tnFaxEntry;
		protected Gtk.Entry tnIPPhoneEntry;
		protected Gtk.Entry ozTitleEntry;
		protected Gtk.Entry ozDeptEntry;
		protected Gtk.Entry ozCompanyEntry;
		protected Gtk.Button smbSetExpireButton;
		protected Gtk.Button smbSetCanButton;
		protected Gtk.Button smbSetMustButton;

		
		public EditUserViewDialog(Connection conn, LdapEntry le)
		{
			Stetic.Gui.Build(this, typeof(lat.EditUserViewDialog));
			
			this.conn = conn;
			this.currentEntry = le;
			
			_memberOfGroups = new Dictionary<string,string> ();
			_allGroups = new Dictionary<string,LdapEntry> ();
			_allGroupGids = new Dictionary<string,string> ();
			_modsGroup = new Dictionary<string,LdapModification> ();
			
			TreeViewColumn col;

			_memberOfStore = new ListStore (typeof (string));
			memberOfTreeview.Model = _memberOfStore;
			memberOfTreeview.Selection.Mode = SelectionMode.Multiple;

			col = memberOfTreeview.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;
	
			_memberOfStore.SetSortColumnId (0, SortType.Ascending);
			
			isSamba = Util.CheckSamba (currentEntry);
			if (!isSamba)
				firstTimeSamba = true;

			try { GetGroups (currentEntry); }
			catch (Exception e) { Log.Debug (e); }

			string userName = conn.Data.GetAttributeValueFromEntry (currentEntry, "cn");
			this.Title = userName + " Properties";

			// General
			usernameLabel.UseMarkup = true;
			usernameLabel.Markup = 
				String.Format ("<span size=\"larger\"><b>{0}</b></span>", conn.Data.GetAttributeValueFromEntry (currentEntry, "uid"));

			fullnameLabel.Text = String.Format ("{0} {1}", 
				conn.Data.GetAttributeValueFromEntry (currentEntry, "givenName"),
				conn.Data.GetAttributeValueFromEntry (currentEntry, "sn"));

			firstNameEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "givenName");
			initialsEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "initials");
			lastNameEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "sn");
			descriptionEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "description");
			officeEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "physicalDeliveryOfficeName");
			mailEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "mail");
			phoneEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "telephoneNumber");

			// Account
			usernameEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "uid");
			uidSpinButton.Value = int.Parse (conn.Data.GetAttributeValueFromEntry (currentEntry, "uidNumber"));
			shellEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "loginShell");;
			homeDirEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "homeDirectory");

			if (isSamba) {
				ToggleSambaWidgets (true);
				smbEnableSambaButton.Hide ();

				smbLoginScriptEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "sambaLogonScript");
				smbProfilePathEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "sambaProfilePath");
				smbHomePathEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "sambaHomePath");
				smbHomeDriveEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "sambaHomeDrive");
								
				smbExpireEntry.Text = GetStringDate (conn.Data.GetAttributeValueFromEntry (currentEntry, "sambaKickoffTime"));
				smbCanChangePwdEntry.Text = GetStringDate (conn.Data.GetAttributeValueFromEntry (currentEntry, "sambaPwdCanChange"));
				smbMustChangePwdEntry.Text = GetStringDate (conn.Data.GetAttributeValueFromEntry (currentEntry, "sambaPwdMustChange"));

			} else {

				smbEnableSambaButton.Toggled += new EventHandler (OnSambaChanged);
				ToggleSambaWidgets (false);
			}

			// Groups
			string pgid = conn.Data.GetAttributeValueFromEntry (currentEntry, "gidNumber");
			if (_allGroupGids.ContainsKey (pgid))
				primaryGroupLabel.Text = _allGroupGids [pgid];

			// Address
			adStreetTextView.Buffer.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "street");
			adPOBoxEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "postOfficeBox");
			adCityEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "l");
			adStateEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "st");
			adZipEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "postalCode");

			// Telephones
			tnHomeEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "homePhone");
			tnPagerEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "pager");
			tnMobileEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "mobile");
			tnFaxEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "facsimileTelephoneNumber");

			// Organization
			ozTitleEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "title");
			ozDeptEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "departmentNumber");
			ozCompanyEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "o");

			this.Run ();

			while (missingValues || errorOccured) {

				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();				
			}

			this.Destroy ();			
		}

		#region internal_methods
		
		bool CheckMemberOf (string user, string[] members)
		{
			foreach (string s in members)
				if (s.Equals (user))
					return true;
	
			return false;			
		}
		
		LdapEntry CreateEntry (string dn)
		{
			LdapAttributeSet aset = new LdapAttributeSet();	
			
			// General
			aset.Add (new LdapAttribute ("cn", fullnameLabel.Text));
			aset.Add (new LdapAttribute ("displayName", fullnameLabel.Text));
			aset.Add (new LdapAttribute ("gecos", fullnameLabel.Text));
			aset.Add (new LdapAttribute ("givenName", firstNameEntry.Text));
			aset.Add (new LdapAttribute ("initials", initialsEntry.Text));
			aset.Add (new LdapAttribute ("sn", lastNameEntry.Text));
			aset.Add (new LdapAttribute ("description", descriptionEntry.Text));
			aset.Add (new LdapAttribute ("physicalDeliveryOfficeName", officeEntry.Text));
			aset.Add (new LdapAttribute ("mail", mailEntry.Text));
			aset.Add (new LdapAttribute ("telephoneNumber", phoneEntry.Text));

			// Account
			aset.Add (new LdapAttribute ("uid", usernameEntry.Text));
			aset.Add (new LdapAttribute ("uidNumber", uidSpinButton.Value.ToString()));
			aset.Add (new LdapAttribute ("homeDirectory", homeDirEntry.Text));
			aset.Add (new LdapAttribute ("loginShell", shellEntry.Text));

			if (passChanged)
				aset.Add (new LdapAttribute ("userPassword", pass));
			else {
				aset.Add (new LdapAttribute ("userPassword", conn.Data.GetAttributeValueFromEntry (currentEntry, "userPassword")));
			}

			if (smbEnableSambaButton.Active || isSamba) {

				aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "posixaccount", "shadowaccount","inetorgperson", "person", "sambaSAMAccount"}));
				
				int user_rid = Convert.ToInt32 (uidSpinButton.Value) * 2 + 1000;
				LdapAttribute[] tmp = Util.CreateSambaAttributes (user_rid, smbSID, smbLM, smbNT);
				foreach (LdapAttribute a in tmp)
					aset.Add (a);
			
				aset.Add (new LdapAttribute ("sambaProfilePath", smbProfilePathEntry.Text));
				aset.Add (new LdapAttribute ("sambaHomePath", smbHomePathEntry.Text));
				aset.Add (new LdapAttribute ("sambaHomeDrive", smbHomeDriveEntry.Text));
				aset.Add (new LdapAttribute ("sambaLogonScript", smbLoginScriptEntry.Text));
				
				double d = 0;
				if (smbExpireEntry.Text != "") {
					d = Util.GetDateTime (smbExpireEntry.Text);
					aset.Add (new LdapAttribute ("sambaKickoffTime", d.ToString()));
				}

				if (smbCanChangePwdEntry.Text != "") {
					d = Util.GetDateTime (smbCanChangePwdEntry.Text);
					aset.Add (new LdapAttribute ("sambaPwdCanChange", d.ToString()));
				}

				if (smbMustChangePwdEntry.Text != "") {
					d = Util.GetDateTime (smbMustChangePwdEntry.Text);
					aset.Add (new LdapAttribute ("sambaPwdMustChange", d.ToString()));
				}
					
			} else {
			
				aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "posixaccount", "shadowaccount","inetorgperson", "person"}));
			}
								
			// Groups
			string gid = GetGidNumber(primaryGroupLabel.Text);
			if (gid != null) {
				aset.Add (new LdapAttribute ("gidNumber", gid));
			} else {
				LdapAttribute la = currentEntry.getAttribute ("gidNumber");
				aset.Add (new LdapAttribute ("gidNumber", la.StringValue));
			}

			// Address
			aset.Add (new LdapAttribute ("street", adStreetTextView.Buffer.Text));
			aset.Add (new LdapAttribute ("l", adCityEntry.Text));
			aset.Add (new LdapAttribute ("st", adStateEntry.Text));
			aset.Add (new LdapAttribute ("postalCode", adZipEntry.Text));
			aset.Add (new LdapAttribute ("postOfficeBox", adPOBoxEntry.Text));

			// Telephones
			aset.Add (new LdapAttribute ("facsimileTelephoneNumber", tnFaxEntry.Text));
			aset.Add (new LdapAttribute ("pager", tnPagerEntry.Text));
			aset.Add (new LdapAttribute ("mobile", tnMobileEntry.Text));
			aset.Add (new LdapAttribute ("homePhone", tnHomeEntry.Text));
			aset.Add (new LdapAttribute ("ipPhone", tnIPPhoneEntry.Text));			

			// Organization
			aset.Add (new LdapAttribute ("title", ozTitleEntry.Text));
			aset.Add (new LdapAttribute ("departmentNumber", ozDeptEntry.Text));
			aset.Add (new LdapAttribute ("o", ozCompanyEntry.Text));				
					
			LdapEntry newEntry = new LdapEntry (dn, aset);
			return newEntry;
		}
		
		string GetGidNumber (string name)
		{
			if (name == null || name == string.Empty)
				return null;

			if (_allGroups.ContainsKey (name)) {
				LdapEntry le = (LdapEntry) _allGroups [name];		
				LdapAttribute attr = le.getAttribute ("gidNumber");
				if (attr != null)
					return attr.StringValue;
			}

			return null;
		}
		
		void GetGroups (LdapEntry le)
		{
			LdapEntry[] grps = conn.Data.SearchByClass ("posixGroup");

			foreach (LdapEntry e in grps) {

				LdapAttribute nameAttr, gidAttr;
				nameAttr = e.getAttribute ("cn");
				gidAttr = e.getAttribute ("gidNumber");

				if (le != null && nameAttr != null) {

					LdapAttribute a;
					a  = e.getAttribute ("memberUid");
					
					if (a != null) {

						if (CheckMemberOf (conn.Data.GetAttributeValueFromEntry (currentEntry, "uid"), a.StringValueArray)
						   && !_memberOfGroups.ContainsKey (nameAttr.StringValue)) {

							_memberOfGroups.Add (nameAttr.StringValue,"memeberUid");
							_memberOfStore.AppendValues (nameAttr.StringValue);
						}
					}
				}

				if (!_allGroups.ContainsKey (nameAttr.StringValue))
					_allGroups.Add (nameAttr.StringValue, e);

				if (gidAttr != null)
					if (!_allGroupGids.ContainsKey (gidAttr.StringValue))
						_allGroupGids.Add (gidAttr.StringValue, nameAttr.StringValue);
			}
				
		}
		
		string GetStringDate (string unixTime)
		{
			try {
			
				double d = double.Parse (unixTime);
				DateTime dt = Util.GetDateTime (d);
				
				return dt.ToString ();
			
			} catch {
				return "";
			}
		}
		
		void ModifyGroup (LdapEntry groupEntry, LdapModification[] mods)
		{
			if (groupEntry == null)
				return;

			try {
			
				conn.Data.Modify (groupEntry.DN, mods);

			} catch (Exception e) {

				string errorMsg =
					Mono.Unix.Catalog.GetString ("Unable to modify group ") + groupEntry.DN;
				
				errorMsg += "\nError: " + e.Message;
				Util.ShowErrorDialog ("Error", errorMsg);
				Log.Debug (e);
			}
		}
		
		void ToggleSambaWidgets (bool state)
		{
			if (state && firstTimeSamba) {
				string msg = Mono.Unix.Catalog.GetString (
					"You must reset the password for this account in order to set a samba password.");

				HIGMessageDialog dialog = new HIGMessageDialog (
						this,
						0,
						Gtk.MessageType.Info,
						Gtk.ButtonsType.Ok,
						"Setting a samba password",
						msg);

				dialog.Run ();
				dialog.Destroy ();
			}

			smbLoginScriptEntry.Sensitive = state;
			smbProfilePathEntry.Sensitive = state;
			smbHomePathEntry.Sensitive = state;
			smbHomeDriveEntry.Sensitive = state;
			smbExpireEntry.Sensitive = state;
			smbCanChangePwdEntry.Sensitive = state;
			smbMustChangePwdEntry.Sensitive = state;
			smbSetExpireButton.Sensitive = state;
			smbSetCanButton.Sensitive = state;
			smbSetMustButton.Sensitive = state;
		}
		
		void UpdateGroupMembership ()
		{
			Log.Debug ("START updateGroupMembership ()");

			LdapEntry groupEntry = null;
			LdapModification[] mods = new LdapModification [_modsGroup.Count];

			int count = 0;

			foreach (string key in _modsGroup.Keys) {

				Log.Debug ("group: {0}", key);

				LdapModification lm = (LdapModification) _modsGroup[key];
				groupEntry = (LdapEntry) _allGroups [key];

				mods[count] = lm;

				count++;
			}	

			ModifyGroup (groupEntry, mods);

			Log.Debug ("END updateGroupMembership ()");
		}
		
		#endregion

		void OnSambaChanged (object o, EventArgs args)
		{
			if (smbEnableSambaButton.Active) {

				smbSID = conn.Data.GetLocalSID ();

				if (smbSID == null) {
					Util.DisplaySambaSIDWarning (this);
					smbEnableSambaButton.Active = false;
					return;
				}

				ToggleSambaWidgets (true);
			} else {

				ToggleSambaWidgets (false);
			}
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			LdapEntry entry = CreateEntry (currentEntry.DN);				 
			LdapEntryAnalyzer lea = new LdapEntryAnalyzer ();
			lea.Run (currentEntry, entry);
				 
			if (lea.Differences.Length == 0)
				return;
			 	
			if (!Util.ModifyEntry (conn, entry.DN, lea.Differences))
				errorOccured = true;
		}

		protected virtual void OnSetPasswordClicked(object sender, System.EventArgs e)
		{
			PasswordDialog pd = new PasswordDialog ();
			pd.Run ();
			
			if (pd.UnixPassword.Equals (""))
				return;

			pass = pd.UnixPassword;
			smbLM = pd.LMPassword;
			smbNT = pd.NTPassword;

			passChanged = true;
			pd.Destroy ();
		}

		protected virtual void OnSetExpireClicked(object sender, System.EventArgs e)
		{
			TimeDateDialog td = new TimeDateDialog ();
			td.Run ();
			if (td.UnixTime != 0) {
				DateTime dt = Util.GetDateTime (td.UnixTime);			
				smbExpireEntry.Text = dt.ToString ();
			}
			
			td.Destroy ();
		}

		protected virtual void OnSetCanClicked(object sender, System.EventArgs e)
		{
			TimeDateDialog td = new TimeDateDialog ();
			td.Run ();
			if (td.UnixTime != 0) {
				DateTime dt = Util.GetDateTime (td.UnixTime);
				smbCanChangePwdEntry.Text = dt.ToString ();
			}
			td.Destroy ();
		}

		protected virtual void OnSetMustClicked(object sender, System.EventArgs e)
		{
			TimeDateDialog td = new TimeDateDialog ();
			td.Run();
			if (td.UnixTime != 0) {
				DateTime dt = Util.GetDateTime (td.UnixTime);
				smbMustChangePwdEntry.Text = dt.ToString ();
			}
			td.Destroy ();
		}

		protected virtual void OnAddGroupClicked(object sender, System.EventArgs e)
		{
			List<string> tmp = new List<string> ();
	
			foreach (KeyValuePair<string, LdapEntry> kvp in _allGroups) {
				if (kvp.Key == primaryGroupLabel.Text || _memberOfGroups.ContainsKey (kvp.Key))
					continue;

				tmp.Add (kvp.Key);
			}

			SelectGroupsDialog sgd = new SelectGroupsDialog (tmp.ToArray ());
			sgd.Run ();

			foreach (string name in sgd.SelectedGroupNames) {

				_memberOfStore.AppendValues (name);
		
				if (!_memberOfGroups.ContainsKey (name))
					_memberOfGroups.Add (name, "memberUid");

				LdapAttribute attr = new LdapAttribute ("memberUid", conn.Data.GetAttributeValueFromEntry (currentEntry, "uid"));
				LdapModification lm = new LdapModification (LdapModification.ADD, attr);

				_modsGroup.Add (name, lm);

				UpdateGroupMembership ();

				_modsGroup.Clear ();
			}
			
			sgd.Destroy ();
		}

		protected virtual void OnRemoveGroupClicked(object sender, System.EventArgs e)
		{
			TreeModel model;
			TreeIter iter;
			
			TreePath[] tp = memberOfTreeview.Selection.GetSelectedRows (out model);

			for (int i  = tp.Length; i > 0; i--) {

				_memberOfStore.GetIter (out iter, tp[(i - 1)]);

				string name = (string) _memberOfStore.GetValue (iter, 0);

				_memberOfStore.Remove (ref iter);
		
				if (_memberOfGroups.ContainsKey (name))
					_memberOfGroups.Remove (name);

				LdapAttribute attr = new LdapAttribute ("memberUid", conn.Data.GetAttributeValueFromEntry (currentEntry, "uid"));
				LdapModification lm = new LdapModification (LdapModification.DELETE, attr);

				_modsGroup.Add (name, lm);
			
				UpdateGroupMembership ();

				_modsGroup.Clear ();
			}
		}

		protected virtual void OnSetPrimaryGroupClicked(object sender, System.EventArgs e)
		{
			List<string> tmp = new List<string> ();
	
			foreach (KeyValuePair<string, LdapEntry> kvp in _allGroups) {

				if (kvp.Key == primaryGroupLabel.Text)
					continue;

				tmp.Add (kvp.Key);
			}

			SelectGroupsDialog sgd = new SelectGroupsDialog (tmp.ToArray());
			sgd.Run ();

			if (sgd.SelectedGroupNames.Length > 0)
				primaryGroupLabel.Text = sgd.SelectedGroupNames[0];
				
			sgd.Destroy ();				
		}

		protected virtual void OnNameChanged(object sender, System.EventArgs e)
		{
			usernameLabel.Markup = 
				String.Format ("<span size=\"larger\"><b>{0}</b></span>", usernameEntry.Text);

			if (initialsEntry.Text != "")
				fullnameLabel.Text = String.Format ("{0} {1} {2}", firstNameEntry.Text, initialsEntry.Text, lastNameEntry.Text);
			else
				fullnameLabel.Text = String.Format ("{0} {1}", firstNameEntry.Text, lastNameEntry.Text);
		}
	}
}
