// 
// lat - GroupsViewDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using Novell.Directory.Ldap;

namespace lat
{	
	public class GroupsViewDialog : Gtk.Dialog
	{
		protected Connection conn;
		protected bool missingValues;
		protected bool errorOccured;
		protected string defaultNewContainer;		

		ListStore allUserStore;
		ListStore currentMemberStore;
		LdapEntry currentEntry;		
		
		bool isEdit;
		bool isSamba;		
		string smbSID = "";
		List<string> currentMembers = new List<string> ();
		
		protected Gtk.Entry groupNameEntry;
		protected Gtk.Entry descriptionEntry;
		protected Gtk.SpinButton groupIDSpinButton;
		protected Gtk.CheckButton enableSambaButton;
		protected Gtk.TreeView allUsersTreeview;
		protected Gtk.TreeView currentMembersTreeview;

		
		public GroupsViewDialog(Connection connection, string newContainer)
		{
			Stetic.Gui.Build(this, typeof(lat.GroupsViewDialog));
			
			this.conn = connection;
			this.defaultNewContainer = newContainer;
			
			Init ();
			PopulateUsers ();
						
			groupIDSpinButton.Value = conn.Data.GetNextGID ();
			this.Title = "Add Group";

			this.Run ();

			while (missingValues || errorOccured) {
				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();				
			}

			this.Destroy ();			
			
		}

		public GroupsViewDialog(Connection connection, LdapEntry le)
		{
			Stetic.Gui.Build(this, typeof(lat.GroupsViewDialog));
			
			this.conn = connection;
			
			currentEntry = le;			
			isEdit = true;
			isSamba = CheckSamba (currentEntry);
			
			Init ();
			
			string groupName = conn.Data.GetAttributeValueFromEntry (currentEntry, "cn");

			this.Title = groupName + " Properties";
			groupNameEntry.Text = groupName;
			descriptionEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "description");
			
			try {
				groupIDSpinButton.Value = int.Parse (conn.Data.GetAttributeValueFromEntry (currentEntry, "gidNumber"));
			} catch {}

			LdapAttribute attr = currentEntry.getAttribute ("memberuid");
			if (attr != null) {
				foreach (string s in attr.StringValueArray) {
					currentMemberStore.AppendValues (s);
					currentMembers.Add (s);
				}
			}

			PopulateUsers ();

			this.Run ();

			while (missingValues || errorOccured){
				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();
			}

			this.Destroy ();
			
		}

		#region internal_methods
		
		bool CheckSamba (LdapEntry le)
		{
			bool retVal = false;
			
			LdapAttribute la = le.getAttribute ("objectClass");
			
			if (la == null)
				return retVal;

			foreach (string s in la.StringValueArray)
				if (s.ToLower() == "sambagroupmapping")
					retVal = true;

			return retVal;
		}
		
		LdapEntry CreateEntry (string dn)
		{
			LdapAttributeSet aset = new LdapAttributeSet();			
			aset.Add (new LdapAttribute ("cn", groupNameEntry.Text));
			aset.Add (new LdapAttribute ("description", descriptionEntry.Text));
			aset.Add (new LdapAttribute ("gidNumber", groupIDSpinButton.Value.ToString()));
			
			if (currentMembers.Count >= 1)
				aset.Add (new LdapAttribute ("memberuid", currentMembers.ToArray()));
						
			if (enableSambaButton.Active || isSamba) {
			
				aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "posixGroup", "sambaGroupMapping"}));
				aset.Add (new LdapAttribute ("sambaGroupType", "2"));

				int grid = Convert.ToInt32 (groupIDSpinButton.Value) * 2 + 1001;
				smbSID = conn.Data.GetLocalSID ();				
				aset.Add (new LdapAttribute ("sambaSID", String.Format ("{0}-{1}", smbSID, grid)));

			} else {
				aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "posixGroup"}));
			}
					
			LdapEntry newEntry = new LdapEntry (dn, aset);
			return newEntry;
		}
		
		void Init ()
		{
			TreeViewColumn col;

			allUserStore = new ListStore (typeof (string));
			allUsersTreeview.Model = allUserStore;
			allUsersTreeview.Selection.Mode = SelectionMode.Multiple;

			col = allUsersTreeview.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;
	
			allUserStore.SetSortColumnId (0, SortType.Ascending);
			
			currentMemberStore = new ListStore (typeof (string));
			currentMembersTreeview.Model = currentMemberStore;
			currentMembersTreeview.Selection.Mode = SelectionMode.Multiple;

			col = currentMembersTreeview.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;
	
			currentMemberStore.SetSortColumnId (0, SortType.Ascending);

			if (isSamba)
				enableSambaButton.Hide ();
			else
				enableSambaButton.Toggled += OnSambaChanged;

			this.Resize (350, 400);		
		}
		
		void PopulateUsers ()
		{
			LdapEntry[] _users = conn.Data.SearchByClass ("posixAccount");

			foreach (LdapEntry le in _users) {
				LdapAttribute nameAttr = le.getAttribute ("uid");
				if (nameAttr != null && !currentMembers.Contains (nameAttr.StringValue) )
					allUserStore.AppendValues (nameAttr.StringValue);
			}					
		}
		
		#endregion

		void OnSambaChanged (object o, EventArgs args)
		{
			if (enableSambaButton.Active) {
				smbSID = conn.Data.GetLocalSID ();

				if (smbSID == null) {
					Util.DisplaySambaSIDWarning (this);
					enableSambaButton.Active = false;
					return;
				}
			}
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			LdapEntry entry = null;
			
			if (isEdit) {
				 entry = CreateEntry (currentEntry.DN);				 
				 
				 LdapEntryAnalyzer lea = new LdapEntryAnalyzer ();
				 lea.Run (currentEntry, entry);
				 
				 if (lea.Differences.Length == 0)
				 	return;
				 	
				 if (!Util.ModifyEntry (conn, entry.DN, lea.Differences))
				 	errorOccured = true;
				 	
			} else {
			
				string userDN = null;
				
				if (this.defaultNewContainer == string.Empty) {
				
					SelectContainerDialog scd =	new SelectContainerDialog (conn);
					scd.Title = "Save Group";
					scd.Message = String.Format ("Where in the directory would\nyou like save the group\n{0}?", groupNameEntry.Text);
					scd.Run ();

					if (scd.DN == "")
						return;

					userDN = String.Format ("cn={0},{1}", groupNameEntry.Text, scd.DN);
					
					scd.Destroy ();
				
				} else {
				
					userDN = String.Format ("cn={0},{1}", groupNameEntry.Text, this.defaultNewContainer);
				}
				
				entry = CreateEntry (userDN);

				string[] missing = LdapEntryAnalyzer.CheckRequiredAttributes (conn, entry);
				if (missing.Length != 0) {
					Util.ShowMissingAttrbutesAlert (missing);
					missingValues = true;
					return;
				}

				if (!Util.AddEntry (conn, entry))
					errorOccured = true;
			}		
		}

		protected virtual void OnAddClicked(object sender, System.EventArgs e)
		{
			TreeModel model;
			TreeIter iter;

			TreePath[] tp = allUsersTreeview.Selection.GetSelectedRows (out model);

			for (int i  = tp.Length; i > 0; i--) {

				allUserStore.GetIter (out iter, tp[(i - 1)]);

				string user = (string) allUserStore.GetValue (iter, 0);
				
				currentMembers.Add (user);				
				currentMemberStore.AppendValues (user);
				
				allUserStore.Remove (ref iter);
			}
		}

		protected virtual void OnRemoveClicked(object sender, System.EventArgs e)
		{
			TreeModel model;
			TreeIter iter;

			TreePath[] tp = currentMembersTreeview.Selection.GetSelectedRows (out model);

			for (int i  = tp.Length; i > 0; i--) {

				currentMemberStore.GetIter (out iter, tp[(i - 1)]);

				string user = (string) currentMemberStore.GetValue (iter, 0);

				currentMemberStore.Remove (ref iter);
		
				if (currentMembers.Contains (user))
					currentMembers.Remove (user);

				allUserStore.AppendValues (user);
			}
		}
	}
}
