// 
// lat - HostsViewDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using Novell.Directory.Ldap;

namespace lat
{	
	public class HostsViewDialog : Gtk.Dialog
	{
		protected LdapEntry currentEntry;
		protected  bool isEdit;
		protected Connection conn;
		protected bool missingValues;
		protected bool errorOccured;
		protected string defaultNewContainer;		
		
		protected Gtk.Entry hostNameEntry;
		protected Gtk.Entry ipEntry;
		protected Gtk.Entry descriptionEntry;

		
		public HostsViewDialog(Connection connection, string newContainer)
		{
			Stetic.Gui.Build(this, typeof(lat.HostsViewDialog));
			
			this.conn = connection;
			this.defaultNewContainer = newContainer;
			this.Title = "Add computer";
			this.Run ();
			
			while (missingValues || errorOccured) {
				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();
			}

			this.Destroy ();			
		}

		public HostsViewDialog(Connection connection, LdapEntry le)
		{
			Stetic.Gui.Build(this, typeof(lat.HostsViewDialog));
			
			this.conn = connection;
			this.currentEntry = le;
			
			isEdit = true;
			currentEntry = le;
			
			string hostName = conn.Data.GetAttributeValueFromEntry (currentEntry, "cn"); 
			this.Title = hostName + " Properties";
			hostNameEntry.Text = hostName;
			
			ipEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "ipHostNumber");
			descriptionEntry.Text = conn.Data.GetAttributeValueFromEntry (currentEntry, "description");

			this.Run ();
			this.Destroy ();			
		}

		#region internal_methods
		
		LdapEntry CreateEntry (string dn)
		{
			LdapAttributeSet aset = new LdapAttributeSet();
			aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "ipHost", "device"}));
			aset.Add (new LdapAttribute ("cn", hostNameEntry.Text));
			aset.Add (new LdapAttribute ("ipHostNumber", ipEntry.Text));
			aset.Add (new LdapAttribute ("description", descriptionEntry.Text));
			
			LdapEntry newEntry = new LdapEntry (dn, aset);
			return newEntry;
		}
		
		#endregion

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			LdapEntry entry = null;
			
			if (isEdit) {
				 entry = CreateEntry (currentEntry.DN);
				 
				 LdapEntryAnalyzer lea = new LdapEntryAnalyzer ();
				 lea.Run (currentEntry, entry);
				 
				 if (lea.Differences.Length == 0)
				 	return;
				 	
				 if (!Util.ModifyEntry (conn, entry.DN, lea.Differences))
				 	errorOccured = true;
				 	
			} else {
			
				SelectContainerDialog scd = new SelectContainerDialog (conn);
				scd.Title = "Save Host";
				scd.Message = String.Format ("Where in the directory would\nyou like save the host\n{0}?", hostNameEntry.Text);
				scd.Run ();

				if (scd.DN == "")
					return;

				string userDN = String.Format ("cn={0},{1}", hostNameEntry.Text, scd.DN);
				entry = CreateEntry (userDN);
				
				scd.Destroy ();

				string[] missing = LdapEntryAnalyzer.CheckRequiredAttributes (conn, entry);
				if (missing.Length != 0) {
					Util.ShowMissingAttrbutesAlert (missing);
					missingValues = true;
					return;
				}

				if (!Util.AddEntry (conn, entry))
					errorOccured = true;
			}		
		}
	}
}
