// 
// lat - NewUserViewDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Text;
using Mono.Security.Protocol.Ntlm;
using Novell.Directory.Ldap;

namespace lat
{	
	public class NewUserViewDialog : Gtk.Dialog
	{
		protected Connection conn;
		protected bool missingValues;
		protected bool errorOccured;
		protected string defaultNewContainer;
		
		Dictionary<string,LdapEntry> _allGroups;
		Dictionary<string,string> _allGroupGids;
		Dictionary<string,string> _memberOfGroups;

		string smbSID = "";
		string smbLM = "";
		string smbNT = "";

		ComboBox primaryGroupComboBox;
		Dictionary<string,string> defaultValues;
	
		protected Gtk.Label usernameLabel;
		protected Gtk.Label fullnameLabel;
		protected Gtk.Entry firstNameEntry;
		protected Gtk.Entry initialsEntry;
		protected Gtk.Entry lastNameEntry;
		protected Gtk.Entry displayNameEntry;
		protected Gtk.Entry usernameEntry;
		protected Gtk.SpinButton uidSpinButton;
		protected Gtk.HBox groupHBox;
		protected Gtk.Entry passwordEntry;
		protected Gtk.Entry homeDirEntry;
		protected Gtk.Entry shellEntry;
		protected Gtk.CheckButton enableSambaButton;

		
		public NewUserViewDialog(Connection connection, string newContainer, Dictionary<string,string> defaultValues)
		{
			Stetic.Gui.Build(this, typeof(lat.NewUserViewDialog));
			
			this.conn = connection;
			this.defaultNewContainer = newContainer;
			this.defaultValues = defaultValues;
			
			_memberOfGroups = new Dictionary<string,string> ();
			_allGroups = new Dictionary<string,LdapEntry> ();
			_allGroupGids = new Dictionary<string,string> ();
			
			GetGroups ();
			CreateCombo ();
			
			SetDefaults ();
			
			uidSpinButton.Value = conn.Data.GetNextUID ();
			
			// FIXME: can't set in designer for some reason
			enableSambaButton.Toggled += OnSambaChanged;
			
			this.Run ();

			while (missingValues || errorOccured) {
				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();				
			}

			this.Destroy ();
		}

		#region internal_methods
		
		void CreateCombo ()
		{
			if (primaryGroupComboBox != null) {
				primaryGroupComboBox.Changed -= OnPrimaryGroupChanged;
				primaryGroupComboBox.Destroy ();
				primaryGroupComboBox = null;
			}
			
			primaryGroupComboBox = ComboBox.NewText ();
			
			string defaultGroup = "None";
			if (defaultValues != null && defaultValues.ContainsKey ("defaultGroup")) {
				defaultGroup = defaultValues["defaultGroup"];
				primaryGroupComboBox.AppendText (defaultGroup);
			}

			foreach (string key in _allGroups.Keys) {
				if (key.ToLower() == defaultGroup.ToLower())
					continue;
					
				primaryGroupComboBox.AppendText (key);
			}

			primaryGroupComboBox.AppendText ("Create new group...");

			primaryGroupComboBox.Active = 0;
			primaryGroupComboBox.Changed += OnPrimaryGroupChanged;
			primaryGroupComboBox.Show ();

			groupHBox.PackStart (primaryGroupComboBox, true, true, 0);
		}
		
		LdapEntry CreateEntry (string dn)
		{
			LdapAttributeSet aset = new LdapAttributeSet();	

			TreeIter iter;
				
			if (primaryGroupComboBox.GetActiveIter (out iter)) {
				string pg = (string) primaryGroupComboBox.Model.GetValue (iter, 0);
				aset.Add (new LdapAttribute ("gidNumber", GetGidNumber(pg)));
			}
						
			aset.Add (new LdapAttribute ("givenName", firstNameEntry.Text));
			aset.Add (new LdapAttribute ("sn", lastNameEntry.Text));
			aset.Add (new LdapAttribute ("uid", usernameEntry.Text));
			aset.Add (new LdapAttribute ("uidNumber", uidSpinButton.Value.ToString()));
			aset.Add (new LdapAttribute ("userPassword", passwordEntry.Text));
			aset.Add (new LdapAttribute ("loginShell", shellEntry.Text));
			aset.Add (new LdapAttribute ("homeDirectory", homeDirEntry.Text));
			aset.Add (new LdapAttribute ("displayName", displayNameEntry.Text));
			aset.Add (new LdapAttribute ("cn", displayNameEntry.Text));
			aset.Add (new LdapAttribute ("gecos", displayNameEntry.Text));
			
			if (initialsEntry.Text != "")
				aset.Add (new LdapAttribute ("initials", initialsEntry.Text));

			if (enableSambaButton.Active) {

				aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "posixaccount", "shadowaccount","inetorgperson", "person", "sambaSAMAccount"}));
				
				int user_rid = Convert.ToInt32 (uidSpinButton.Value) * 2 + 1000;
				LdapAttribute[] tmp = Util.CreateSambaAttributes (user_rid, smbSID, smbLM, smbNT);
				foreach (LdapAttribute a in tmp)
					aset.Add (a);					

			} else {
			
				aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "posixaccount", "shadowaccount","inetorgperson", "person"}));
			}
					
			LdapEntry newEntry = new LdapEntry (dn, aset);
			return newEntry;
		}
		
		string GetGidNumber (string name)
		{
			if (name == null)
				return null;

			LdapEntry le = (LdapEntry) _allGroups [name];		
			LdapAttribute attr = le.getAttribute ("gidNumber");

			if (attr != null)
				return attr.StringValue;
			
			return null;
		}
		
		void GetGroups ()
		{
			LdapEntry[] grps = conn.Data.SearchByClass ("posixGroup");

			foreach (LdapEntry e in grps) {

				LdapAttribute nameAttr, gidAttr;
				nameAttr = e.getAttribute ("cn");
				gidAttr = e.getAttribute ("gidNumber");

				_allGroups.Add (nameAttr.StringValue, e);
				_allGroupGids.Add (gidAttr.StringValue, nameAttr.StringValue);
			}
				
		}
		
		bool IsPasswordEmpty ()
		{
			if (passwordEntry.Text == "" || passwordEntry.Text == null) {
				string msg = Mono.Unix.Catalog.GetString (
					"You must set a password for the new user");

				Util.ShowErrorDialog ("User error", msg);
				return true;
			}
			
			return false;
		}
		
		bool IsUIDAvailable ()
		{
			if (!Util.CheckUID (conn, Convert.ToInt32 (uidSpinButton.Value))) {
				string msg = Mono.Unix.Catalog.GetString (
					"The UID you have selected is already in use!");

				Util.ShowErrorDialog ("User error", msg);
				return false;
			}
			
			return true;
		}
		
		bool IsUserNameAvailable ()
		{
			if (!Util.CheckUserName (conn, usernameEntry.Text)) {
				string format = Mono.Unix.Catalog.GetString (
					"A user with the username '{0}' already exists!");

				string msg = String.Format (format, usernameEntry.Text);
				Util.ShowErrorDialog ("User error", msg);

				return false;
			}
			
			return true;
		}
		
		void ModifyGroup (LdapEntry groupEntry, LdapModification[] mods)
		{
			if (groupEntry == null)
				return;

			try {
				conn.Data.Modify (groupEntry.DN, mods);

			} catch (Exception e) {

				string errorMsg =
					Mono.Unix.Catalog.GetString ("Unable to modify group ") + groupEntry.DN;

				errorMsg += "\nError: " + e.Message;
				
				Log.Debug (e);
				Util.ShowErrorDialog ("Modify error", errorMsg);
			}
		}
		
		void SetDefaults ()
		{
			if (defaultValues == null)
				return;
		
			if (defaultValues.ContainsKey ("userPassword"))
				passwordEntry.Text = defaultValues ["userPassword"]; 

			if (defaultValues.ContainsKey ("sambaLMPassword"))
				smbLM = defaultValues["sambaLMPassword"];
				
			if (defaultValues.ContainsKey ("sambaNTPassword"))
				smbNT = defaultValues["sambaNTPassword"];				
		
			if (defaultValues.ContainsKey ("loginShell"))
				shellEntry.Text = defaultValues ["loginShell"]; 				

			if (defaultValues.ContainsKey ("enableSamba")) {
				bool enableSamba = bool.Parse (defaultValues ["enableSamba"]);
				enableSambaButton.Active = enableSamba;
			}
		}
		
		void UpdateGroupMembership ()
		{
			LdapEntry groupEntry = null;
			LdapModification[] mods = new LdapModification [1];

			foreach (string key in _memberOfGroups.Keys) {

				LdapAttribute attr = new LdapAttribute ("memberUid", usernameEntry.Text);
				LdapModification lm = new LdapModification (LdapModification.ADD, attr);

				groupEntry = (LdapEntry) _allGroups[key];

				mods[0] = lm;
			}

			ModifyGroup (groupEntry, mods);
		}
		
		#endregion 

		void OnSambaChanged (object o, EventArgs args)
		{
			if (enableSambaButton.Active) {
				smbSID = conn.Data.GetLocalSID ();

				if (smbSID == null) {
					Util.DisplaySambaSIDWarning (this);
					enableSambaButton.Active = false;
					return;
				}
			}
		}

		void OnPrimaryGroupChanged (object o, EventArgs args)
		{
			ComboBox combo = o as ComboBox;
			if (o == null)
				return;
				
			TreeIter iter;
			if (combo.GetActiveIter (out iter)) {
				string selection = (string) combo.Model.GetValue (iter, 0);				
				if (selection == "Create new group...") {
					new GroupsViewDialog (conn, "");
					
					_allGroups.Clear();
					_allGroupGids.Clear();
					GetGroups ();					
					CreateCombo ();
				}
			}
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			LdapEntry entry = null;
			string userDN = null;

			if (!IsUserNameAvailable() || !IsUIDAvailable() || IsPasswordEmpty()) {
				errorOccured = true;
				return;
			}
			
			if (this.defaultNewContainer == null) {
			
				SelectContainerDialog scd =	new SelectContainerDialog (conn);
				scd.Title = "Save group";
				scd.Message = String.Format ("Where in the directory would you like save the user {0}", displayNameEntry.Text);
				scd.Run ();

				if (scd.DN == "")
					return;

				userDN = String.Format ("cn={0},{1}", displayNameEntry.Text, scd.DN);
			
				scd.Destroy ();
				
			} else {
			
				userDN = String.Format ("cn={0},{1}", displayNameEntry.Text, this.defaultNewContainer);
			}
			
			entry = CreateEntry (userDN);

			string[] missing = LdapEntryAnalyzer.CheckRequiredAttributes (conn, entry);
			if (missing.Length != 0) {
				Util.ShowMissingAttrbutesAlert (missing);
				missingValues = true;
				return;
			}

			UpdateGroupMembership ();

			if (!Util.AddEntry (conn, entry))
				errorOccured = true;		
		}

		protected virtual void OnSetPasswordClicked(object sender, System.EventArgs e)
		{
			PasswordDialog pd = new PasswordDialog ();
			pd.Run ();

			if (!passwordEntry.Text.Equals ("") && pd.UnixPassword.Equals (""))
				return;

			passwordEntry.Text = pd.UnixPassword;
			smbLM = pd.LMPassword;
			smbNT = pd.NTPassword;
			
			pd.Destroy ();
		}

		protected virtual void OnDisplayNameFocusIn(object o, Gtk.FocusInEventArgs args)
		{
			string suid = Util.SuggestUserName (
					firstNameEntry.Text, 
					lastNameEntry.Text);

			usernameEntry.Text = suid;

			if (displayNameEntry.Text != "")
				return;

			if (initialsEntry.Text.Equals("")) {
				displayNameEntry.Text = String.Format ("{0} {1}", 
					firstNameEntry.Text, 
					lastNameEntry.Text);
			} else {

				String format = "";
				if (initialsEntry.Text.EndsWith("."))
					format = "{0} {1} {2}";
				else
					format = "{0} {1}. {2}";

				displayNameEntry.Text = String.Format (format, 
					firstNameEntry.Text, 
					initialsEntry.Text, 
					lastNameEntry.Text);
			}

			if (homeDirEntry.Text.Equals("") && !usernameEntry.Text.Equals("")) {
				string defaultDir = "/home";
				if (defaultValues != null && defaultValues.ContainsKey ("homeDirectory"))
					defaultDir = defaultValues["homeDirectory"];
				
				homeDirEntry.Text = System.IO.Path.Combine (defaultDir, usernameEntry.Text); 
			}		
		}

		protected virtual void OnNameChanged(object sender, System.EventArgs e)
		{
			usernameLabel.Markup = 
				String.Format ("<span size=\"larger\"><b>{0}</b></span>", usernameEntry.Text);

			if (initialsEntry.Text != "")
				fullnameLabel.Text = String.Format ("{0} {1} {2}", firstNameEntry.Text, initialsEntry.Text, lastNameEntry.Text);
			else
				fullnameLabel.Text = String.Format ("{0} {1}", firstNameEntry.Text, lastNameEntry.Text);			
		}
	}
}
