// 
// lat - UserDefaultValuesDialog.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using Novell.Directory.Ldap;

namespace lat
{	
	public class UserDefaultValuesDialog : Gtk.Dialog
	{
		ComboBox primaryGroupComboBox;
		ViewPlugin vp;
		Connection conn;
		
		string smbLM = "";
		string smbNT = "";
		
		protected Gtk.HBox groupHBox;
		protected Gtk.Entry passwordEntry;
		protected Gtk.Entry homeEntry;
		protected Gtk.Entry shellEntry;
		protected Gtk.CheckButton sambaCheckButton;

		
		public UserDefaultValuesDialog(ViewPlugin plugin, Connection connection)
		{
			Stetic.Gui.Build(this, typeof(lat.UserDefaultValuesDialog));
			
			SetDefaultValues ();
		}

		#region internal_methods
		
		void CreateCombo (string defaultGroup)
		{
			if (primaryGroupComboBox != null) {
				primaryGroupComboBox.Destroy ();
				primaryGroupComboBox = null;
			}
			
			primaryGroupComboBox = ComboBox.NewText ();
			primaryGroupComboBox.AppendText (defaultGroup);

			LdapEntry[] grps = conn.Data.SearchByClass ("posixGroup");
			foreach (LdapEntry e in grps) {
				LdapAttribute nameAttr = e.getAttribute ("cn");
				if (nameAttr.StringValue.ToLower() == defaultGroup.ToLower())
					continue;
					
				primaryGroupComboBox.AppendText (nameAttr.StringValue);
			}

			primaryGroupComboBox.Active = 0;
			primaryGroupComboBox.Show ();

			groupHBox.PackStart (primaryGroupComboBox, true, true, 0);
		}
		
		void SetDefaultValues ()
		{
			Log.Debug ("vp.PluginConfiguration.Defaults.Count: {0}", vp.PluginConfiguration.Defaults.Count);
		
			if (vp.PluginConfiguration.Defaults.ContainsKey ("userPassword"))
				passwordEntry.Text = vp.PluginConfiguration.Defaults ["userPassword"]; 

			if (vp.PluginConfiguration.Defaults.ContainsKey ("sambaLMPassword"))
				smbLM = vp.PluginConfiguration.Defaults["sambaLMPassword"];
				
			if (vp.PluginConfiguration.Defaults.ContainsKey ("sambaNTPassword"))
				smbNT = vp.PluginConfiguration.Defaults["sambaNTPassword"];				

			if (vp.PluginConfiguration.Defaults.ContainsKey ("homeDirectory"))
				homeEntry.Text = vp.PluginConfiguration.Defaults ["homeDirectory"]; 
				
			if (vp.PluginConfiguration.Defaults.ContainsKey ("loginShell"))
				shellEntry.Text = vp.PluginConfiguration.Defaults ["loginShell"]; 				

			if (vp.PluginConfiguration.Defaults.ContainsKey ("enableSamba")) {
				bool enableSamba = bool.Parse (vp.PluginConfiguration.Defaults ["enableSamba"]);
				sambaCheckButton.Active = enableSamba;
			}

			if (vp.PluginConfiguration.Defaults.ContainsKey ("defaultGroup"))
				CreateCombo (vp.PluginConfiguration.Defaults["defaultGroup"]);
			else
				CreateCombo ("None");
		}
		
		#endregion

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			TreeIter iter;
			if (primaryGroupComboBox.GetActiveIter (out iter)) {
				string pg = (string) primaryGroupComboBox.Model.GetValue (iter, 0);
				if (pg != "None")
					vp.PluginConfiguration.Defaults["defaultGroup"] = pg;
			}
			
			if (passwordEntry.Text != "") {
				vp.PluginConfiguration.Defaults["userPassword"] = passwordEntry.Text;
				if (sambaCheckButton.Active) {
					vp.PluginConfiguration.Defaults["sambaLMPassword"] = smbLM;
					vp.PluginConfiguration.Defaults["sambaNTPassword"] = smbNT;
				}
			}

			if (homeEntry.Text != "")
				vp.PluginConfiguration.Defaults["homeDirectory"] = homeEntry.Text;
			
			if (shellEntry.Text != "")
				vp.PluginConfiguration.Defaults["loginShell"] = shellEntry.Text;
			
			vp.PluginConfiguration.Defaults["enableSamba"] = sambaCheckButton.Active.ToString();
			
			Log.Debug ("vp.PluginConfiguration.Defaults.Count: {0}", vp.PluginConfiguration.Defaults.Count);		
		}

		protected virtual void OnSetPasswordClicked(object sender, System.EventArgs e)
		{
			PasswordDialog pd = new PasswordDialog ();
			pd.Run ();

			if (!passwordEntry.Text.Equals ("") && pd.UnixPassword.Equals (""))
				return;

			passwordEntry.Text = pd.UnixPassword;
			smbLM = pd.LMPassword;
			smbNT = pd.NTPassword;
						
			pd.Destroy ();
		}
	}
}
